@@ -9,7 +9,7 @@ from rest_framework.response import Response
 from .models import Metadata
 from .permissions import *
 from .serializers import *
-from .utils import get_204_wrapped_response, get_wrapped_response, valid_str
+from .utils import get_204_wrapped_response, get_wrapped_response, valid_str, is_http_response_success
 
 
 class SecurityQuestionView(viewsets.ModelViewSet):
@@ -19,8 +19,9 @@ class SecurityQuestionView(viewsets.ModelViewSet):
 
 
 class ProfileView(generics.RetrieveUpdateDestroyAPIView):
+    lookup_field = USER_LOOKUP_FIELD
     queryset = get_user_model().objects.all()
-    serializer_class = get_serializer_class()
+    serializer_class = profile_serializer_class
     permission_classes = [IsOwnerOrSuperuserOrReadOnly, ]
 
     def get(self, request, *args, **kwargs):
@@ -42,7 +43,21 @@ class SignUpView(generics.CreateAPIView):
     permission_classes = [permissions.AllowAny, ]
 
     def post(self, request, *args, **kwargs):
-        return get_wrapped_response(super().post(request, *args, **kwargs))
+        request_data = request.data
+        username = request_data.get('username', request_data.get('email', request_data.get('mobile_number', None)))
+
+        response = super().post(request, *args, **kwargs)
+        if is_http_response_success(status_code=response.status_code):
+            user = get_user_model().objects.filter(
+                Q(username=username) | Q(email=username) | Q(mobile_number=username),
+            ).first()
+            self.on_success(user)
+        return get_wrapped_response(response)
+
+    @staticmethod
+    def on_success(user):
+        if user and user.requires_verification:
+            token, code = user.request_verification(send_mail=True)
 
 
 class SignInView(views.APIView):
@@ -69,7 +84,7 @@ class SignOutView(views.APIView):
         return get_wrapped_response(Response({'success': 'signed out'}, status=status.HTTP_200_OK))
 
 
-class VerificationCodeRequestView(views.APIView):
+class VerificationRequestView(views.APIView):
     permission_classes = [permissions.IsAuthenticated, ]
     serializer_class = AuthTokenOnlySerializer
 
@@ -81,7 +96,7 @@ class VerificationCodeRequestView(views.APIView):
         return get_wrapped_response(response)
 
 
-class VerificationCodeVerifyView(VerificationCodeRequestView):
+class VerificationConfirmView(VerificationRequestView):
     """
     Attempts to verify authenticated user's verification code(retrieved from a POST request using
     'code' as key).
@@ -96,10 +111,10 @@ class VerificationCodeVerifyView(VerificationCodeRequestView):
 
     def post(self, request, format=None):
         user = request.user
-        operation = request.query_params.get('operation', 'verify').lower()
+        operation = request.query_params.get('operation', 'confirm').lower()
         if re.match('^(re(send|quest)|send)$', operation):
             # new verification code resend or request is been made
-            return super(VerificationCodeVerifyView, self).post(request, format)
+            return super(VerificationConfirmView, self).post(request, format)
         else:
             # verify provided code
             code = request.data.get('code', None)
@@ -132,7 +147,7 @@ class PasswordResetRequestView(views.APIView):
         return get_wrapped_response(Response(data, status=status_code))
 
 
-class PasswordResetView(PasswordResetRequestView):
+class PasswordResetConfirmView(PasswordResetRequestView):
     """
     Attempts to reset(change) authenticated user's password(retrieved from a POST request using
     `temporary_password` as key).
@@ -153,7 +168,7 @@ class PasswordResetView(PasswordResetRequestView):
         if re.match('^(re(send|quest)|send)$', operation):
             # probably a new request for password reset
             # get username to resend the email
-            return super(PasswordResetView, self).post(request, format)
+            return super(PasswordResetConfirmView, self).post(request, format)
         else:
             # reset password
             t_pass = data.get('temporary_password', data.get('old_password', None))
@@ -193,7 +208,7 @@ class AddSecurityQuestionView(views.APIView):
         return get_wrapped_response(response)
 
 
-class AccountActivationRequestView(views.APIView):
+class ActivationRequestView(views.APIView):
     """
     Provides a user with possible account activation methods provided a correct account
     `username` is supplied as part of a POST request.
@@ -202,7 +217,8 @@ class AccountActivationRequestView(views.APIView):
     serializer_class = AuthTokenOnlySerializer
 
     def post(self, request, format=None):
-        username = request.data.get('username', None)
+        request_data = request.data
+        username = request_data.get('username', request_data.get('email', None))
         user = request.user
         is_valid_user = user and not isinstance(user, AnonymousUser)
         user = user if is_valid_user else get_user_model().objects.filter(
@@ -227,7 +243,7 @@ class AccountActivationRequestView(views.APIView):
         return True if (metadata and metadata.security_question.usable) else False
 
 
-class AccountActivationView(AccountActivationRequestView):
+class ActivationConfirmView(ActivationRequestView):
     """
     Activates a user's account when provided by a correct `answer` in a POST request with a `answer` as key.
     """
@@ -235,7 +251,7 @@ class AccountActivationView(AccountActivationRequestView):
 
     def post(self, request, format=None):
         user, data = request.user, request.data
-        operation = request.query_params.get('operation', 'activate').lower()
+        operation = request.query_params.get('operation', 'confirm').lower()
         if re.match('^request$', operation):
             # probably a new request for password reset
             # get username to resend the email
